unit maintcsvdocu;
// Test mit TCSVDocument
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, EditBtn,
  StdCtrls;

type

  TDatensatz = record
    IDNr: Integer;
    LfdNr: Integer;
    Zahl: Double;
    Datum: TDate;
    Text: String;
    Betrag: Double;
  end;

  TDatenArray = array of TDatensatz;
                  (*
  TDatenSatz = record
    IDNR : string[3];  // Numerisch
    LfdNr: string[7];  // Numerisch
    Zahl : string[19]; // Numerisch 17,2
    Datum: string[10]; // TT.MM.JJJJ
    Text : string[20];
    Betrag: string[19]; // Numerisch 17,2
  end;
                    *)

  { TForm1 }

  TForm1 = class(TForm)
    btnClose: TButton;
    btnStart: TButton;
    FileDialog: TFileNameEdit;
    lblFileDialog: TLabel;
    memLog: TMemo;
    procedure btnCloseClick(Sender: TObject);
    procedure btnStartClick(Sender: TObject);

  private
    Data: TDatenArray;
    procedure LeseDatei(const AFileName: String);
    procedure Ausgabe;

  public

  end;



var
  Form1: TForm1;

implementation

uses
  StrUtils, csvreadwrite;

{$R *.lfm}

function ThSepStrToFloat(const NumberStr: String; Thousandsep: Char): Double;
var
  i, j: Integer;
  s: String;
begin
  SetLength(s, Length(NumberStr));
  j := 0;
  for i:= 1 to Length(NumberStr) do
    if NumberStr[i] <> ThousandSep then begin
      inc(j);
      s[j] := NumberStr[i];
    end;
  SetLength(s, j);
  Result := StrToFloat(s);
end;


{ TForm1 }

procedure TForm1.btnStartClick(Sender: TObject);
begin
   if FileDialog.FileName = '' then begin
     MessageDlg('Keine CSV Datei ausgewählt',mtError,[mbOK],0);
     FileDialog.SetFocus;
     exit;
   end;
  LeseDatei(FileDialog.FileName);
  Ausgabe;
end;

procedure TForm1.LeseDatei(const AFileName: String);
const
  BLOCK_SIZE = 1000;
var
  counter: Integer;
  csv: TCsvParser;
  stream: TFileStream;
begin
  SetLength(data, 0);
  // Counter zählt die eingelesenen Zeilen
  counter := 0;
  csv := TCSVParser.Create;
  try
    csv.Delimiter := ';';
    stream := TFileStream.Create(AFileName, fmOpenRead + fmShareDenyWrite);
    try
      csv.SetSource(stream);
      while csv.ParseNextCell do begin
        if csv.CurrentRow <> counter then   // Neue Zeile erreicht
        begin
          inc(counter);
          if ((counter - 1) mod BLOCK_SIZE = 0) then
            // Das Datenarray wird in "Häppchen" der Größe BLOCK_SIZE dimensioniert
            SetLength(data, Length(data) + BLOCK_SIZE);
        end;
        if counter > 0 then
          with data[counter-1] do   // -1, um die Titelzeile zu überspringen
            case csv.CurrentCol of
              0: IDNr := StrToInt(csv.CurrentCellText);
              1: LfdNr := StrToInt(csv.CurrentCellText);
              2: Zahl := ThSepStrToFloat(csv.CurrentCellText, FormatSettings.ThousandSeparator);
              3: Datum := StrToDate(csv.CurrentCelLText);
              4: Text := Uppercase(csv.CurrentCellText);
              5: Betrag := ThSepStrToFloat(csv.CurrentCellText, FormatSettings.ThousandSeparator);
            end;
      end;
      SetLength(data, counter);
    finally
      stream.Free;
    end;
  finally
    csv.Free;
  end;
end;

// Damit man etwas sieht, das eingelesene Daten-Array im Memo ausgeben
procedure TForm1.Ausgabe;
var
  s: String;
  i: Integer;
begin
  memLog.Lines.BeginUpdate;
  try
    memLog.Lines.Clear;
    s := PadLeft('ID', 3) + ' ' + PadLeft('LfdNr', 7) + ' ' +PadLeft('Zahl', 19) + ' ' +
         PadLeft('Datum', 15) + '   ' + PadRight('Text', 10) + ' ' + PadLeft('Betrag', 19);
    memLog.Lines.Add(s);
    for i := 0 to High(Data) do begin
      with Data[i] do
        s := Format('%3d %7d %19.2f %15s   %-10s %19.2f', [
          IDNr, LfdNr, Zahl, DateToStr(Datum), Text, Betrag
        ]);
      memLog.Lines.Add(s);
    end;
  finally
    memLog.Lines.EndUpdate;
  end;
end;

procedure TForm1.btnCloseClick(Sender: TObject);
begin
  Close;
end;


end.

